/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package funambol.syncclient.blackberry.email.impl;
import java.util.Hashtable;
import java.util.Vector;
import net.rim.blackberry.api.mail.Message;
import net.rim.blackberry.api.mail.Folder;
import net.rim.blackberry.api.mail.Store;
import net.rim.blackberry.api.mail.Session;
import net.rim.device.api.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import net.rim.blackberry.api.mail.Multipart;
import net.rim.blackberry.api.mail.SupportedAttachmentPart;
import net.rim.blackberry.api.mail.TextBodyPart;
import funambol.syncclient.common.Base64;
import net.rim.blackberry.api.mail.FolderNotFoundException;
import net.rim.blackberry.api.mail.Address;
import funambol.syncclient.util.StaticDataHelper;

/**
 * <code>MIMEParser</code> To parse MIME content in SyncML.
 *
 * $Id: MIMEParser.java,v 1.0
 */
public class MIMEParser {
 
/**
* 
* This method processes the MIME data & returns the message object
* 
*/     
 public Message parseMessage(String messageString,String locURI){
        
        Message msg     =   new Message();
        
        Hashtable ht = new Hashtable();
        
        try{
            
            messageString = extractMIMEContent(messageString);
            
            ht = CreateHashTable(messageString);
            
            MailContent mailBody$Attachment=new MailContent((String) ht.get("CONTENT-TYPE"));
            
            mailBody$Attachment.parseContent();
            
            MailBody body = mailBody$Attachment.getMailBody();
            
            Vector attachment = mailBody$Attachment.getAttachmentList();
            
            msg.addHeader("FROM",(String)ht.get("FROM"));
            
            msg.addHeader("TO",(String)ht.get("TO"));
            
            msg.addHeader("Date",(String)ht.get("DATE"));
            
            msg.addHeader("MIME-Version",(String)ht.get("MIME-VERSION"));
            
            MessageIDStore msgIdStore = new MessageIDStore();
            
            msgIdStore.setMessageMappingKey();
            
            msgIdStore.add(locURI,(String)ht.get("MESSAGE-ID"));
            
            msg.setSubject((String)ht.get("SUBJECT"));
            
            if((String)ht.get("FROM") != null){
                
                String strFrom = (String)ht.get("FROM");
                
                if(strFrom.indexOf("<") >= 0){
                    
                    strFrom = strFrom.substring(strFrom.indexOf("<") + 1,strFrom.indexOf(">"));
                }
                
                Address from = new Address(strFrom.trim(), " ");
                
                msg.setFrom(from);
            }
            
            if((String)ht.get("TO") != null){
                
                String strTo = (String)ht.get("TO");
                
                if(strTo.indexOf("<") >= 0){
                    
                    strTo = strTo.substring(strTo.indexOf("<") + 1,strTo.indexOf(">"));
                
                }
                
                Address to[] = new Address[1];
                
                to[0]= new Address(strTo.trim(), " ");
                
                msg.addRecipients(Message.RecipientType.TO, to);
            }
            
           if(attachment.size() == 0){
                             
                if(body.getActualContent() != null){
                    
                    String encoding = (String)ht.get("CONTENT-TRANSFER-ENCODING");
                
                    if(encoding != null && encoding.trim().equalsIgnoreCase("base64")){
                        
                        msg.setContent(new String(Base64.decode(body.getActualContent().getBytes())));
                    
                    }else{
                        
                        msg.setContent(body.getActualContent());
                    }
                }
                   
             }else{
                 
                 Multipart mp = new Multipart();
                 
                 TextBodyPart bodypart = new TextBodyPart(mp);
                 
                 int bodyPartIndex = 0;
                 
                 if(body.getActualContent() != null){
                    
                    String encoding = body.getContentTransferEncoding();
                    
                    bodypart.setContentType(body.getContentType());
                    
                    String content = null;
                        
                    if(encoding != null && encoding.trim().equalsIgnoreCase("base64")){
                        
                        content = new String(Base64.decode(body.getActualContent().getBytes()));
                    
                    }else{
                        
                        content = body.getActualContent();
                    
                    }
                    
                    bodypart.setContent(content);
                          
                    mp.addBodyPart(bodypart,bodyPartIndex);
                    
                    ++bodyPartIndex;
                }
                
                for(int i = 0; i < attachment.size(); i++){
                    
                    SupportedAttachmentPart sp =  new SupportedAttachmentPart(mp);
                    
                    MailAttachment mailAttachment = (MailAttachment)attachment.elementAt(i);
                    
                    sp.setFilename(mailAttachment.getFileName());
                     
                    if(mailAttachment.getContentTransferEncoding().equals("base64")){
                        
                        sp.setContent(new String(Base64.decode(mailAttachment.getactualContent().getBytes())));
                    
                    }else{
                        
                        sp.setContent(mailAttachment.getactualContent());
                    }
                    
                    sp.setContentType(mailAttachment.getContentType());
                    
                    mp.addBodyPart(sp,bodyPartIndex);
                    
                    ++bodyPartIndex;
                 }
                
                 msg.setContent(mp); 
            }
        
            
            }catch(FolderNotFoundException f){
                
                StaticDataHelper.log(f.getMessage());
            
            }catch(Exception e){
                
                StaticDataHelper.log(e.getMessage());
            
            }
         
            return msg;
        
    }
    
    /**
    * 
    * Helper method to parse the data, this method populates the Hashtable & returns it
    * 
    */  
    public Hashtable CreateHashTable(String msg) throws Exception {
    
        String boundary = "''~~";//Intialise boundary to some junk value
        
        Hashtable htable = new Hashtable();
        
        String[] splitVals = new StringUtil().split(msg,"\n");
        
            for (int i = 0; i < splitVals.length; i++) {
                
                String temp = splitVals[i];
                
                if (temp.trim().equals(boundary)) {
                    
                    StringBuffer section = new StringBuffer();
                    
                    for (int j = i; j < splitVals.length; j++){
                        
                        section.append(splitVals[j]).append("\n");
                    
                    }
                    
                    String contentType = (String) htable.get("CONTENT-TYPE");
                    
                    if (contentType != null){
                        
                        htable.put("CONTENT-TYPE", contentType + section.toString());
                    }
                    
                    break;
                }
                
                if(temp.trim().length()==0){
                    
                    if(StringUtil.removeWideSpaces(temp).equals("\r") 
                        || StringUtil.removeWideSpaces(temp).equals("\r\r")){
                        
                        String contentType=new String();
                        
                        if(htable.containsKey("CONTENT-TYPE")){
                        
                            contentType=(String)htable.get("CONTENT-TYPE");
                        }
                            
                        String helper=contentType.toLowerCase();
                        
                        if((contentType!=null) 
                            && ((contentType.trim().length() != 0)) 
                            && (helper.indexOf("multipart/related")== -1) 
                            && (helper.indexOf("multipart/mixed")== -1)){
                            
                            StringBuffer section=new StringBuffer();
                            
                            for(int j=i;j<splitVals.length;j++){    
                                
                                section.append(splitVals[j]).append("\n");
                            
                            }
                                
                            if(contentType!=null){
                                
                                htable.put("CONTENT-TYPE",contentType+section.toString());
                            
                            }
                            
                            break;
                        }
                    }
                }
                
                String[] keyValue = new StringUtil().split(temp,":");
                
                if (keyValue.length == 2) {
                
                    String key = keyValue[0].toUpperCase().trim();
                    
                    String value = keyValue[1];
                    
                    if((key.equalsIgnoreCase("Content-Disposition")) 
                        && (splitVals[i+1].toLowerCase().indexOf("filename") >= 0)){
                        
                        String[] file = new StringUtil().split(splitVals[i+1],"=");
                        
                        htable.put("FILENAME", file[1]);
                        
                        ++i;
                    }
                    
                    if (!key.equalsIgnoreCase("Content-Type")) {
                        
                        String storedValue = (String) htable.get(key);
                    
                        if (storedValue != null) {
                            
                            value = storedValue + "[``-<==>-``]" + value;
                    
                        }
                        
                        htable.put(key.trim(), value);
                    
                    } else {
                        
                            if (value.equals("*/*\r")) {
                                
                                htable.put("CONTENT-TYPE", value + "\n");
                            
                            } else if (!(value.indexOf("multipart/related") < 0)) {
                                
                                i = i + 1;
                            
                                htable.put("CONTENT-TYPE", value + "\n" + splitVals[i]+ "\n");
                                
                                boundary = "--" + findBoundary(splitVals[i]) ;
                            
                            } else if (!(value.indexOf("multipart/mixed") < 0)) {
                                
                                i = i + 1;
                                
                                htable.put("CONTENT-TYPE", value + "\n" + splitVals[i]+ "\n");
                                
                                boundary = "--" + findBoundary(splitVals[i]) ;
                            
                            } else{
                                
                                if(splitVals[i+1].indexOf("charset=") >= 0) {
                                    
                                    value = value+splitVals[i+1].trim();
                                    
                                    ++i;
                                }
                                
                                htable.put("CONTENT-TYPE", value + "\r\n");
                            }
                        
                    }
                
                } else if (keyValue.length > 2) {
                    
                    String value = new String();
                    
                    String key = keyValue[0].toUpperCase();
                    
                    for (int tempi = 1; tempi < keyValue.length; tempi++){
                        value = value + ":" + keyValue[tempi];
                    }
                    
                    value = value.substring(1, value.length());
                    
                    htable.put(key.trim(), value);
                }
            }
            
        return htable;
    }
    
    /**
    * 
    * This method find the boundry in the MIME data
    * 
    */    
    public static String findBoundary(String content) {
        
        String copy = content.toLowerCase();
        
        int startIndex = copy.indexOf("boundary=\"");
        
        int endIndex = copy.indexOf("\"",startIndex+10);
        
        if (startIndex >= 0 && endIndex >= 0){
            
            return content.substring(startIndex+10, endIndex);
        }
        
        startIndex = copy.indexOf("boundary=");
        
        endIndex = copy.indexOf(";");
        
        if(endIndex==-1){
            
            endIndex = copy.indexOf("\r");
        
        }
        
        if (startIndex > 0 && endIndex > 0){
            
            return content.substring(startIndex + 9, endIndex).trim();
        }
        
        return null;
    }
    
    /**
    * 
    * This method extracts the required MIME data from the message
    * 
    */  
      
    private String extractMIMEContent(String content){
        
        String DATA_START_MARKER    = "<![CDATA[";
        
        String DATA_END_MARKER      = "]]&gt;";
        
        String mimeData = null;
        
        int startMarkerIndex = content.indexOf(DATA_START_MARKER);
        
        int endMarkerIndex = content.indexOf(DATA_END_MARKER);
        
        if(startMarkerIndex == -1 || endMarkerIndex == -1) {
            
            return null;
        
        }
        
        return mimeData = content.substring((startMarkerIndex 
                    + DATA_START_MARKER.length()),endMarkerIndex);
    }
      
}
