/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package funambol.syncclient.blackberry.email.impl;

import funambol.syncclient.util.StaticDataHelper;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;

/**
 * <code>MessageIDStore</code> class for storing and retrieving message id's.
 *
 * $Id: MessageIDStore.java,v 1.0
 */
public class MessageIDStore {
    
        
    private long KEY;
    private static PersistentObject persist;
    public MessageIDStore() { 
        KEY = Constants.FOLDERITEMS;
    }
    
   /**
    * sets the passed key for store 
    */    
    public void setMessageMappingKey(){
        
        this.KEY=Constants.MSGMAPKEY;
    
    }   
    
   /**
    * adds new  servId:clientId to store 
    * @param   String  :servId   passed server id 
    * @param   String  :clientId passed client id
    */      
    public void add(String servId, String clientId) {
    
        if(servId == null ||  clientId == null){
            
            return;
        
        }
        
        String data = servId.trim() + ":" + clientId.trim();
        
        if(checkIsDataThere(data)){
            
            return;
        
        }
        
        persist = PersistentStore.getPersistentObject( KEY );
        
        String valuesFromStore[] = (String [] ) persist.getContents();
                
        if(valuesFromStore == null){
            
            valuesFromStore  = new String [1];
            valuesFromStore[0] =new String(data);
            persist.setContents(valuesFromStore);
            persist.commit();
            return;
        
        }
        else{
            
            String IncrementedStore []= new String[valuesFromStore.length+1];
            
            int i=0;
            
            for(i=0;i<valuesFromStore.length;i++){
                
                IncrementedStore[i]=valuesFromStore[i];
            
            }
            
            IncrementedStore[i]=data;
            
            persist.setContents(IncrementedStore);
            
            persist.commit();
        } 

    }       
  
   
   /**
    * finds if data is present in Store 
    * @param String  :data this the data to be searched 
    * @return boolean :true if data is present else false
    */  
    
  
    public boolean checkIsDataThere(String data){
        
        boolean isThere=false;
        persist = PersistentStore.getPersistentObject( KEY );
        String valuesFromStore[] = (String [] ) persist.getContents();
        
        if(valuesFromStore != null){
        
            for(int i=0;i<valuesFromStore.length;i++){
                
                String[] compare = new StringUtil().split(valuesFromStore[i],":");
                
                if((compare != null) && (compare[0].equals(data) ||  compare[1].equals(data))){
                        
                        return true;
                
                }
                
            }
        
        }
        return isThere;
    }
    
  /**
    * finds if server id  is present for passed clientId in Store 
    * @param String  :clientID  this the passed client id
    * @return String :serverId if it is present else null
    */  
      
    public String getServerId(String clientID) {
    
        String serverId = null;
        persist = PersistentStore.getPersistentObject( KEY );
        String valuesFromStore[] = (String [] ) persist.getContents();
        if(valuesFromStore != null){
                
                for(int i=0;i<valuesFromStore.length;i++){
                    
                    String[] compare = new StringUtil().split(valuesFromStore[i],":");
                    if(compare[1].equals(clientID)){
                    
                        serverId =compare[0];
                        StaticDataHelper.log("found serverId-->"+serverId);
                
                    }
    
                }
            
        }
           
        return serverId;
    
    }
    
    /**
    * finds if client id  is present for passed serverId in Store 
    * @param String  :serverId  this the passed server id
    * @return String :clientID if it is present else null
    */
     public String getClientId(String serverId) {
    
        String clientID  = null;
        String temp = serverId;
        persist = PersistentStore.getPersistentObject( KEY );
        String valuesFromStore[] = (String [] ) persist.getContents();
        if(valuesFromStore != null){
                        
                for(int i=0;i<valuesFromStore.length;i++){
                
                        String[] compare = new StringUtil().split(valuesFromStore[i],":");
                
                        if(compare[0].equals(serverId)){
                
                            clientID =compare[1];  
                
                            StaticDataHelper.log("found clientID-->"+clientID);
                        }
                    }
            
        }
           
        return clientID;
    }
    
   
   
     /**
    * deletes   data from Store if present
    * @param String  :data this the data to be deleted 
    */  
     
    public void delete(String data){
        
        persist = PersistentStore.getPersistentObject( KEY );
    
        String valuesFromStore[] = (String [] ) persist.getContents();
    
        if(valuesFromStore ==null){
    
            return;
    
        }
        else{
    
            boolean flag = false;
    
            int j =0;
    
            String small [] = new String[valuesFromStore.length];
    
            for(int i=0;i<valuesFromStore.length;i++){
    
                String[] compare = new StringUtil().split(valuesFromStore[i],":");
    
                if(compare != null && !compare[0].equals(data)){
    
                    small[j++]=valuesFromStore[i];
    
                }
                else{
    
                    flag=true; //found
    
                }
            }
            
            if(flag){
                
                if(j>0){
                   
                    String result []  = new String[j];
                    
                    for(int k=0;k<j;k++){
                        
                        result[k]=small[k];
                        
                        StaticDataHelper.log("result   "+result[k]);
                    }
            
                    persist.setContents(result);
                    
                    persist.commit();
                
                }
                else{
                    
                    PersistentStore.destroyPersistentObject(KEY);
                
                }
            }
            else{
                
                persist.setContents(small);
                
                persist.commit();
            
            }
        
        }
    
    }
   
} 


