/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package funambol.syncclient.blackberry.email.impl;

import funambol.syncclient.blackberry.Configuration;
import funambol.syncclient.blackberry.email.impl.MailParser;
import funambol.syncclient.blackberry.email.impl.XMLMailParser;
import funambol.syncclient.blackberry.parser.ParserFactory;
import funambol.syncclient.util.StaticDataHelper;
import net.rim.blackberry.api.mail.Message;
import net.rim.blackberry.api.mail.Folder;
import net.rim.blackberry.api.mail.Store;
import net.rim.blackberry.api.mail.Session;
import java.util.*;
import funambol.syncclient.spds.SyncMLClientImpl;
import funambol.syncclient.blackberry.email.impl.SyncGen;
import funambol.syncclient.spds.SyncException;
import funambol.syncclient.spds.AuthenticationException;
import funambol.syncclient.spds.UpdateException;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import java.io.IOException;
import java.lang.InterruptedException;
import funambol.syncclient.spds.NetworkException;
import net.rim.blackberry.api.mail.Transport;

/**
 * <code>ProcessMail</code> Helper class to process messages
 *
 * $Id: ProcessMail.java,v 1.0
 */
public class ProcessMail {
    
 private static PersistentObject store;
     private static long   CONFIG_KEY     = 0xb5651L;

    public final static int
        ALERT_CODE_ONE_WAY_FROM_SERVER                  = 204             ;
    
    public static final int
        ALERT_CODE_REFRESH_FROM_SERVER                  = 205             ;
    
    public static final int
        ALERT_CODE_SLOW                                 = 201             ;
    
    public static final int
        ALERT_CODE_FAST                                 = 200             ;

    public static final int
        STATUS_CODE_FORBIDDEN                           = 401             ;
    
    public static final int
        STATUS_CODE_OK                                  = 200             ;
    
    public static final int
        STATUS_CODE_AUTHENTICATION_ACCEPTED             = 212             ;
       
    public static final int
        STATUS_REFRESH_REQUIRED                         = 508             ;

    public static final int
        STATUS_CODE_FINE                                = 201             ;
           
    final private static String ERROR_NOT_AUTHORIZED                    = "401"          ;
    final private static String ERROR_NOT_FOUND                         = "404"          ;
    final private static String ERROR_CONNECTION_NOT_FOUND              = "100"          ;
    final private static String ERROR_SERVER                            = "511"          ;
    final private static String ERROR_REFRESH_REQUIRED                  = "508"          ; 
    private static final String TAG_ALERT            = "Alert"         ;
    private static final String TAG_ADD              = "Add"           ;
    private static final String TAG_CMD              = "Cmd"           ;
    private static final String TAG_DATA             = "Data"          ;
    private static final String TAG_DELETE           = "Delete"        ;
    private static final String TAG_ITEM             = "Item"          ;
    private static final String TAG_LOC_URI          = "LocURI"        ;
    private static final String TAG_REPLACE          = "Replace"       ;
    private static final String TAG_STATUS           = "Status"        ;
    private static final String TAG_SYNC             = "Sync"          ;
    private static final String TAG_SYNCBODY         = "SyncBody"      ;
    private static final String TAG_SYNCHDR          = "SyncHdr"       ;
    private static final String TAG_SYNCML           = "SyncML"        ;
    private static final String TAG_TARGET           = "Target"        ;
    
    private static final String DEVICE_ID_PREFIX     = "fbb-"          ;

    private  Hashtable   serverAlerts          = null ;
    private  String      alertXML              = null ;
    private  String      clientInitXML         = null ;
    private  String      modificationsXML      = null ;
    private  String      mappingXML            = null ;
    private  String      mailXML               = null ;
    private  SyncGen      sg;
    private MailParser mailParser              = null ;
    private String responseURI                 = null ;
    private StringBuffer stringStatusBuffer    = null ;
    private long timeStamp                     =     0;
    private String networkURL                  = null ;
    private Vector messageVector               = null ;
    private int count                          =     0;
    private int num_of_count                   =     0;
    private static PersistentObject persist    =  null;
    private String serverTimeStamp             =  null;
    private int  tracker                       =     0;
    private int  msgCount                      =     2;
    private int  msgRefCount                   =     1;
    
    public ProcessMail(){   
    
            mailParser =  ParserFactory.getParserInstance();
    
    }
        
    /**
    * returns  the specific error 
    * @param String  :error this the error message to be logged 
    * @return String :error exact error mesage or error if no match
    */          
    private String getSynchError(String error){
        
        StaticDataHelper.log(error);
        
        if ( error !=null && ERROR_NOT_AUTHORIZED.equals(error)) {
        
            return "not-authorized-user-and-password";
        
        } else if (error != null && ERROR_NOT_FOUND.equals(error)) {
        
            return "not-found-remote-syncsource"    ;                              
        
        
        } else if( error != null &&  error.indexOf("Unknown host") != -1){
        
        
            return "Unknown host";
        
        } else if (error != null &&  error.indexOf("Connection refused") != -1) {
        
            return "error-connecting-to-the-server";
        
        } else if (error != null && error.indexOf("out-of-memory") != -1) {
        
            return "out-of-memory";
        
        } else if (error != null && ERROR_SERVER.equals(error)) {
        
            return " Internal Server Error "    ;         
          
        } else if (error != null && error.indexOf(ERROR_REFRESH_REQUIRED)>=0) {
            
            PersistentStore.destroyPersistentObject(SyncGen.CONFIG_KEY);
            
            return " Refresh Required "    ;         
        
        } else {
        
            return error;
        
        }

    }
    
    
   /**
    * checks if there is any exception in response
    * @param String  :response response string as returned from server 
    * @throws Exception
    */ 
    
      
   private void  checkresponse(String response) throws Exception{
     
        if(response !=null && response.indexOf("Unknown host")!=-1){
                                
            throw new Exception( "Unknown host" ); 
        
        }
       
    }
    
    
   /**
    * finds if data is present in Store 
    * @param String  :data this the data to be searched 
    * @param long  :key this the key for this store
    * @return boolean :true if data is present else false
    */  
    
      
    public boolean checkIsDataThere(String data, long key){
        
        persist = PersistentStore.getPersistentObject( key );
        
        String vals[] = (String [] ) persist.getContents();
        
        if(vals != null){
        
            for(int i=0;i<vals.length;i++){
        
                String[] compare = new StringUtil().split(vals[i],":");
        
                if(compare[1].equals(data)){
        
                    StaticDataHelper.log("found data"+data);
        
                    return true;
                }
            }
        
        }
        
        return false;
    
    }
    
   
   
     /**
    * adds new  data to Store 
    * @param String  :data this the data to be added 
    * @param long  :key this the key for this store
    */  
    
    
    public void appendData(String data, long key){ 
    
        StaticDataHelper.log("inside append");
    
        persist = PersistentStore.getPersistentObject( key );
        
        String vals[] = (String [] ) persist.getContents();
                
        if(vals == null){
    
            vals  = new String [1];
            vals[0] =new String(data);
            persist.setContents(vals);
            persist.commit();
            return;
    
        }
        else{
    
            String large []= new String[vals.length+1];
            
            int i=0;
            
            for(i=0;i<vals.length;i++){
    
                large[i]=vals[i];
    
            }
            
            large[i]=data;
    
            persist.setContents(large);
    
            persist.commit();
    
        }
     }
    
     /**
    * deletes   data from Store if present
    * @param String  :data this the data to be deleted 
    * @param long  :key this the key for this store
    */  
     
    public void deleteData(String data,long key){
        
        persist = PersistentStore.getPersistentObject( key );
    
        String vals[] = (String [] ) persist.getContents();
    
        if(vals ==null){
    
            return;
    
        }
        else{
    
            boolean flag = false;
    
            int j =0;
    
            String small [] = new String[vals.length];
    
            for(int i=0;i<vals.length;i++){
    
                String[] compare = new StringUtil().split(vals[i],":");
    
                if(compare != null && !compare[1].equals(data)){
    
                    small[j++]=vals[i];
    
                }
                else{
    
                    flag=true; //found
    
                }
            }
            
            if(flag){
                
                if(j>0){
    
                    String result []  = new String[j];
    
                    for(int k=0;k<j;k++){
    
                        result[k]=small[k];
        
                    }
            
                    persist.setContents(result);
    
                    persist.commit();
    
                }
                else{
    
                    PersistentStore.destroyPersistentObject(Constants.ITEMSKEY);
    
                }
    
            }
            else{
    
                persist.setContents(small);
                persist.commit();
    
            }
        }
    
    }
   /**
    * send  authentication Message  to server
    * @return int :alertCode on success or -1 
    * @throws AuthenticationException
    * @throws SyncException
    * @throws IOException 
    * @throws Exception
    */  
    
    public int  sendAuthenticationMessage() throws 
      AuthenticationException, UpdateException ,SyncException ,IOException ,Exception {
            
            
            if(!Configuration.GATEWAYAPN.equals("")   
            && !Configuration.GATEWAYIP.equals("") 
            && !Configuration.GATEWAYPORT.equals(""))
            {
            
                networkURL =  ";WAPGatewayIP="+ Configuration.GATEWAYIP + 
                    ";WAPGatewayAPN="+ Configuration.GATEWAYAPN+
                    ";WAPGatewayPort="+ Configuration.GATEWAYPORT; 
            
            
            }else{
            
                networkURL = "";
            
            }
            
           
            String response=null;
            timeStamp = System.currentTimeMillis();
            SyncGen.nextTimeStamp = timeStamp;
            sg = new SyncGen();
            sg.loadResources();
                            
            clientInitXML=sg.getInit();
            StaticDataHelper.log("Authentication Req"+clientInitXML);
            SyncMLClientImpl impl = new SyncMLClientImpl(Configuration.URL+networkURL);
            response = impl.sendMessage(clientInitXML);
                
                            
            //check for server status codes 
            try{ 
                
                checkresponse(response);
                
                responseURI = getXMLTagValue(response,"RespURI");
           
           }catch(Exception e) {
           
                StaticDataHelper.log(getSynchError(e.getMessage()));
           
            }
                
            try{
           
                checkStatus(response, TAG_SYNCHDR);
                checkStatus(response, TAG_ALERT);  
                serverTimeStamp =getXMLTagValue(getXMLTagValue(response,"Alert"),"Next");
           
            }catch (AuthenticationException ae) {
           
                throw ae ;
           
            }catch(UpdateException ue) {
           
                throw ue ;
           
            }
            catch(SyncException se){
           
                throw se ;
           
            }

            checkServerAlerts(response);//stores in serverAlerts hashtable
            
            String alert = (String)serverAlerts.get(Configuration.PROTOCOL);
            if (alert != null){
           
                return getSourceAlertCode(Configuration.PROTOCOL);
           
            }else{ 
           
                return -1;
           
            }
             
        }
    
    /**
    * returns a nested tag data 
    * @param   String  :xml this the xml message to be searched for a tag 
    * @param   String  :tag this is the tag for which substring should be returned
    * @return  String  :between @param <tag> to </tag>
    */  
    
    private String getXMLTagValue(String xml, String tag) {

        String startTag  = null;
        String endTag    = null;

        startTag = "<" + tag + ">";
        endTag   = "</" + tag + ">";

        return xml.substring(xml.indexOf(startTag) + startTag.length(), xml.indexOf(endTag));

    } 
    
     
    /**
    * creates alerts for all deleted messages  
    * @return  String  :deleted messages alert
    */  
    
    private void processAllDeleteMessages(){
             
        persist = PersistentStore.getPersistentObject( Constants.DELETEDITEMS );
        
        String[] deletedMessages = (String[])persist.getContents();
        
      
        
        if(deletedMessages == null){
            
            return;
        
        }
             
        for(int deletedIndex = 0;deletedIndex < deletedMessages.length;deletedIndex++){
            
            String[] message = new StringUtil().split(deletedMessages[deletedIndex],":");
            
            MessageIDStore messageIDStore = new MessageIDStore();
            
            if(!messageIDStore.checkIsDataThere(message[1])){
                
                continue;
            
            }
            StringBuffer messageBuffer = new StringBuffer();
            messageBuffer.append("<Delete>"+"\n");
            messageBuffer.append("<CmdID>"+mailParser.getCommandId()+"</CmdID>"+"\n");
            messageBuffer.append("<Item>"+"\n");
            messageBuffer.append("<Source>"+"\n");
            messageBuffer.append("<LocURI>");   
            String messageIDfromStore = messageIDStore.getServerId(message[1]);
            
            if(messageIDfromStore != null){
                
                messageBuffer.append(messageIDfromStore);
                messageIDStore.delete(messageIDfromStore);
            
            }else{
                
                messageBuffer.append(message[0]+"/"+ message[1]);
            
            }
            
            messageBuffer.append("</LocURI>"+"\n");        
            messageBuffer.append("</Source>"+"\n"); 
            messageBuffer.append("</Item>"+"\n"); 
            messageBuffer.append("</Delete>");
            StaticDataHelper.log("hre-->"+messageBuffer.toString());
            messageVector.addElement(messageBuffer.toString());
            messageBuffer =null;
            deleteData(message[1],Constants.ITEMSKEY);
            tracker++;
            num_of_count++;
            if(tracker>=Constants.MAX_ALLOWED_MSGS)
            {
                if(num_of_count>1)
                mailParser.setCommandId(3);
                else
                mailParser.setCommandId(4);
                tracker =0;
            }
           
        }
        
        PersistentStore.destroyPersistentObject(Constants.DELETEDITEMS);
       
    }
    
    
     /**
    * processes messages from all folders except deleted messages
    * @param   int  :alertCode type of alert as sent by server 
    * @return  String  :syncml for messages corresponding to type of sync
    * @throws Exception
    */  
    
         
    private void getMessagesFromFolder(int alertCode) throws Exception{
             
        messageVector = new Vector();
       
        processAllDeleteMessages();
        
        Store store = Session.waitForDefaultSession().getStore();
        Folder[] folders = store.list();
        
        for (int index = 0; index < folders.length; index++) {
            
            StaticDataHelper.log("************");
            StaticDataHelper.log(folders[index].getName());
            Folder f = folders[index];
           
            Message[] mailMsg = f.getMessages();
            String fname = f.getFullName();
            String folderName = null;   
            
            if((fname.indexOf("Outbox") >= 0)
                || (fname.indexOf("Inbox") >= 0)){
                          
                for(int msgIndex = 0; msgIndex < mailMsg.length;msgIndex++){
                     
                    MessageIDStore messageIDStore = new MessageIDStore();                 
                    
                    if(fname.indexOf("Outbox") >= 0){
                        
                        folderName = "O";
                    
                    }else if(fname.indexOf("Inbox") >= 0 && mailMsg[msgIndex].isSet(Message.Flag.SAVED)){
                        
                        folderName = "D";
                                          
                    }else if(messageIDStore.getServerId(new Integer(mailMsg[msgIndex].getMessageId()).toString()) != null){
                        
                        String messageIDfromStore = messageIDStore.getServerId(new Integer(mailMsg[msgIndex].getMessageId()).toString());
                        
                        if(messageIDfromStore.indexOf("I") >= 0){
                            
                            folderName = "I";
                         
                         }else if(messageIDfromStore.indexOf("T") >= 0){
                            
                            folderName = "T";
                         
                         }else if(messageIDfromStore.indexOf("S") >= 0){
                            
                            folderName = "S";
                         
                         }
                            
                    }else{
                            folderName = "I";
                    }
                        
                    StaticDataHelper.log(mailMsg[msgIndex].getSubject());
                                                            
                   
                    if(ALERT_CODE_FAST == alertCode){
                            
                       String data = doFastSync("Add",folderName,mailMsg[msgIndex]);
                       if(data != null &&  !data.equals("")){
                         messageVector.addElement(data);
                       }
                            
                    }else{
                            
                           String data = doFastSync("Add",folderName,mailMsg[msgIndex]);
                           if(data != null &&  !data.equals("")){
                                messageVector.addElement(data);
                            }
                            
                    }
                   
                }
            }
        }
        
       
    
    }
    
    /**
    * generates syncml for fastSync 
    * @param   String  :command type i.e add , delete etc. 
    * @param   String  :folderName name of folder. 
    * @param   Message :mailMsg mail message to be synced  
    * @return  String  :syncml for message corresponding to type of sync and message
    */   
        
    private String doFastSync(String command, String folderName,Message mailMsg){
        
        StringBuffer buffer = new StringBuffer();
        
        if(!checkIsDataThere((new Integer(mailMsg.getMessageId()).toString()),Constants.ITEMSKEY )){
        
            mailParser.setParseParams(folderName,mailMsg);
            buffer.append(mailParser.toString());
            appendData(folderName+":"+new Integer(mailMsg.getMessageId()).toString(),Constants.ITEMSKEY);
        
        }
            tracker++;
            num_of_count++;
            if(tracker>=Constants.MAX_ALLOWED_MSGS)
            {
                if(num_of_count>1)
                mailParser.setCommandId(3);
                else
                mailParser.setCommandId(4);
                tracker =0;
            }
        
        return buffer.toString();
    
    } 
    
   /**
    * generates syncml for slowSync 
    * @param   String  :command type i.e add , delete etc. 
    * @param   String  :folderName name of folder. 
    * @param   Message :mailMsg mail message to be synced  
    * @return  String  :syncml for message corresponding to type of sync and message
    */    
     
    private String doSlowSync(String command, String folderName,Message mailMsg){
        
        StringBuffer buffer = new StringBuffer();
        mailParser.setParseParams(folderName,mailMsg);
        buffer.append(mailParser.toString());
            
        if(!checkIsDataThere((new Integer(mailMsg.getMessageId()).toString()),Constants.ITEMSKEY )){
                
                appendData(folderName+":"+new Integer(mailMsg.getMessageId()).toString(),Constants.ITEMSKEY);
        
        }
            tracker++;
            num_of_count++;
            if(tracker>=Constants.MAX_ALLOWED_MSGS)
            {
                if(num_of_count>1)
                mailParser.setCommandId(3);
                else
                mailParser.setCommandId(4);
                tracker =0;
            }
        return buffer.toString();
    }  
    
   /**
    * <description>
    * @param name <description>
    * @param request <description>
    * @return <description>
    */
     
     
   private String sendSyncMessage(String name,String request) throws NetworkException,IOException,
                             AuthenticationException, UpdateException, SyncException               
   {
     String response = null;  
     try {
     StaticDataHelper.log("sending---"+name);
     SyncMLClientImpl impl = new SyncMLClientImpl(responseURI+networkURL);
     response = impl.sendMessage(request);
     StaticDataHelper.log("response --"+name);
     StaticDataHelper.log(response);
     //check for server status codes
     checkStatus(response, TAG_SYNCHDR);
     if(name.indexOf("mailXml")>=0){
     checkStatus(response, TAG_SYNC);
    }
    else if(name.indexOf("finalizeMsg1")>=0){
    checkStatus(response, TAG_ALERT);
    }
    }catch(NetworkException ne){StaticDataHelper.log("NetworkException-->"+ne.getMessage()); throw ne;
    }catch(IOException ie){StaticDataHelper.log("IOException--->"+ie.getMessage()); throw ie;
    }catch (AuthenticationException ae) {throw ae ;
    }catch(UpdateException ue) {throw ue ;
    }catch(SyncException se){throw se ;
    }   
    return response; 
 }
   
       
   /**
    * generates syncml for client messages 
    * @param   int  :alertCode type of alert as sent by server 
    * @throws AuthenticationException
    * @throws SyncException
    * @throws UpdateException 
    * @throws IOException
    */
         
   private void sendClientMailSyncMessage(int alertcode)throws Exception{
                       
                       

        String response = null;
        if(-1 == alertcode){
            return;
        }
        
        sg = new SyncGen();
        sg.loadResources();
       
        mailXML = sg.createMailMessage();
        String alertStatusXml= buildAlertStatusMsg();
        alertStatusXml=sg.messageFormat(alertStatusXml, "{3}", Configuration.PROTOCOL  );
        alertStatusXml=sg.messageFormat(alertStatusXml, "{4}", Configuration.PROTOCOL  );
        alertStatusXml=sg.messageFormat(alertStatusXml, "{5}", "<Next>"+serverTimeStamp+"</Next>");
          

        StringBuffer messageString = new StringBuffer();
        String backupMailXml = mailXML;

        if(ALERT_CODE_ONE_WAY_FROM_SERVER == alertcode){
            //only from server
            mailXML =   sg.messageFormat(mailXML, "{8}", "");
            mailXML =   sg.messageFormat(mailXML, "{11}", alertStatusXml);
            mailXML =   sg.messageFormat(mailXML, "{10}", "<Final></Final>"); 
            mailXML =   sg.messageFormat(mailXML, "{12}", msgCount+"");
            mailXML =   sg.messageFormat(mailXML, "{14}", msgRefCount+"");
            mailXML =   sg.messageFormat(mailXML, "{15}", "3");
            msgCount++;
            msgRefCount++;

        }else if(alertcode == ALERT_CODE_SLOW){
            //slow sync
            try{

                getMessagesFromFolder(ALERT_CODE_SLOW);
                Enumeration messageEnum =messageVector.elements();

                if(messageEnum.hasMoreElements())
                {
                while ( messageEnum.hasMoreElements() )
                {
                    String tempMailXml = "";

                    for(int i = 0; i < Constants.MAX_ALLOWED_MSGS && messageEnum.hasMoreElements() ;i++)
                    {
                         Object vectorData = messageEnum.nextElement();
                         tempMailXml +=vectorData.toString();

                    }
                            mailXML = backupMailXml;
                            if(count>=1)
                            {
                              mailXML =   sg.messageFormat(mailXML, "{11}", "");
                              mailXML =   sg.messageFormat(mailXML, "{15}", "2");
                            }
                            else
                            {
                             mailXML =   sg.messageFormat(mailXML, "{11}", alertStatusXml);
                             mailXML =   sg.messageFormat(mailXML, "{15}", "3");
                         
                            }
                            mailXML =   sg.messageFormat(mailXML, "{8}", tempMailXml);
                            mailXML =   sg.messageFormat(mailXML, "{12}", msgCount+"");
                            mailXML =   sg.messageFormat(mailXML, "{14}", msgRefCount+"");
                           
                            if(messageEnum.hasMoreElements()){

                               mailXML =   sg.messageFormat(mailXML, "{10}", "");
                            }
                            else{
                                 mailXML =   sg.messageFormat(mailXML, "{10}", "<Final></Final>");
                            }
                            //send request
                            sendSyncMessage("mailXml",mailXML);
                            count++;
                            msgCount++;
                            msgRefCount++;


                     }

               
            }else{
                       mailXML =   sg.messageFormat(mailXML, "{11}", alertStatusXml);
                       mailXML =   sg.messageFormat(mailXML, "{8}", "");
                       mailXML =   sg.messageFormat(mailXML, "{10}", "<Final></Final>");
                       mailXML =   sg.messageFormat(mailXML, "{12}", msgCount+"");
                       mailXML =   sg.messageFormat(mailXML, "{14}", msgRefCount+"");
                       mailXML =   sg.messageFormat(mailXML, "{15}", "3");
                     
                       sendSyncMessage("mailXml",mailXML);
                       msgCount++;
                       msgRefCount++;
                                           
            }

           }catch(Exception e){

                StaticDataHelper.log(e.getMessage());
                throw e;

           }



        } else if(alertcode == ALERT_CODE_FAST){
            //fast sync
            try{

               getMessagesFromFolder(ALERT_CODE_FAST);
               Enumeration messageEnum =messageVector.elements();
      
                if(messageEnum.hasMoreElements())
                {
                while ( messageEnum.hasMoreElements() )
                {
                    String tempMailXml = "";

                    for(int i = 0; i < Constants.MAX_ALLOWED_MSGS && messageEnum.hasMoreElements() ;i++)
                    {
                         Object vectorData = messageEnum.nextElement();
                         tempMailXml +=vectorData.toString();

                    }
                            mailXML = backupMailXml;
                            if(count>=1)
                            {
                              mailXML =   sg.messageFormat(mailXML, "{11}", "");
                              mailXML =   sg.messageFormat(mailXML, "{15}", "2");
                            }
                            else
                            {
                             mailXML =   sg.messageFormat(mailXML, "{11}", alertStatusXml);
                             mailXML =   sg.messageFormat(mailXML, "{15}", "3");
                         
                            }
                            mailXML =   sg.messageFormat(mailXML, "{8}", tempMailXml);
                            mailXML =   sg.messageFormat(mailXML, "{12}", msgCount+"");
                            mailXML =   sg.messageFormat(mailXML, "{14}", msgRefCount+"");
                           
                            if(messageEnum.hasMoreElements()){

                               mailXML =   sg.messageFormat(mailXML, "{10}", "");
                            }
                            else{
                                 mailXML =   sg.messageFormat(mailXML, "{10}", "<Final></Final>");
                            }
                             sendSyncMessage("mailXml",mailXML);     

                             count++;
                             msgCount++;
                             msgRefCount++;

                     }

               
            }else{
                       mailXML =   sg.messageFormat(mailXML, "{11}", alertStatusXml);
                       mailXML =   sg.messageFormat(mailXML, "{8}", "");
                       mailXML =   sg.messageFormat(mailXML, "{10}", "<Final></Final>");
                       mailXML =   sg.messageFormat(mailXML, "{12}", msgCount+"");
                       mailXML =   sg.messageFormat(mailXML, "{14}", msgRefCount+"");
                       mailXML =   sg.messageFormat(mailXML, "{15}", "3");
                       sendSyncMessage("mailXml",mailXML);
                       msgCount++;
                       msgRefCount++;
                       
            }

            }catch(Exception e){

                StaticDataHelper.log(e.getMessage());
            }



        }else {
                //refresh from server
               
               mailXML =   sg.messageFormat(mailXML, "{8}", "");
               mailXML =   sg.messageFormat(mailXML, "{11}", alertStatusXml);
               mailXML =   sg.messageFormat(mailXML, "{10}", "<Final></Final>"); 
               mailXML =   sg.messageFormat(mailXML, "{12}", msgCount+"");
               mailXML =   sg.messageFormat(mailXML, "{14}", msgRefCount+"");
               mailXML =   sg.messageFormat(mailXML, "{15}", "3");
               
               sendSyncMessage("mailXml",mailXML);
               msgCount++;
               msgRefCount++;
              

     }
    }
    
    
    /**
    * builds status of server commands 
    * @param   String  :cmdRef command refrence of server message
    * @param   String  :cmdType type of command 
    * @param   String  :targetRef
    * @param   String  :statusCode send appropriate status code
    * @return  int     :cmdID current command index in message to be build
    */           
    private void buildStatusMsg(String cmdRef,String cmdType,String targetRef, String statusCode, int cmdID){
    
        stringStatusBuffer.append("<Status>"+"\n");
        stringStatusBuffer.append("<CmdID>"+cmdID+"</CmdID>"+"\n");
        stringStatusBuffer.append("<MsgRef>"+(msgRefCount+1)+"</MsgRef>"+"\n");
        stringStatusBuffer.append("<CmdRef>"+cmdRef+"</CmdRef>"+"\n");                  
        stringStatusBuffer.append("<Cmd>"+cmdType+"</Cmd>"+"\n");   
        
        if(cmdType.equals("Add")) {
        
            stringStatusBuffer.append("<SourceRef>"+targetRef+"</SourceRef>"+"\n");  
        
        }else{
        
            stringStatusBuffer.append("<TargetRef>"+targetRef+"</TargetRef>"+"\n");            
        
        }
        
        stringStatusBuffer.append("<Data>"+statusCode+"</Data>"+"\n");            
        stringStatusBuffer.append("</Status>"+"\n");
    }
    
    
    /**
    * builds status for server first alert 
    * @return  String  : status String
    */     
    
    private String buildAlertStatusMsg(){
   
        StringBuffer  stringStatusBufferAlert = new StringBuffer();  
        stringStatusBufferAlert.append("<Status>"+"\n");
        stringStatusBufferAlert.append("<CmdID>"+2+"</CmdID>"+"\n");
        stringStatusBufferAlert.append("<MsgRef>"+1+"</MsgRef>"+"\n");
        stringStatusBufferAlert.append("<CmdRef>"+3+"</CmdRef>"+"\n");                  
        stringStatusBufferAlert.append("<Cmd>"+"Alert"+"</Cmd>"+"\n");   
        stringStatusBufferAlert.append("<TargetRef>"+"{3}"+"</TargetRef>"+"\n");
        stringStatusBufferAlert.append("<SourceRef>"+"{4}"+"</SourceRef>"+"\n");  
        stringStatusBufferAlert.append("<Data>"+200+"</Data>"+"\n");    
        stringStatusBufferAlert.append("<Item>"); 
        stringStatusBufferAlert.append("<Data>");
        stringStatusBufferAlert.append("<Anchor xmlns='syncml:metinf'>");
        stringStatusBufferAlert.append("{5}");
        stringStatusBufferAlert.append("</Anchor>");  
        stringStatusBufferAlert.append("</Data>");
        stringStatusBufferAlert.append("</Item>");
        stringStatusBufferAlert.append("</Status>"+"\n");
        return stringStatusBufferAlert.toString();
    }
    
    
    /**
    * creates and sends final syncml to server
    * @throws SyncException
    * @throws IOException
    */   
        
    private void finalizeSync() throws SyncException,IOException,NetworkException{
            
        String finalizeMsg1 = sg.getFinalizeMsg1();
        String backupFinalizeMsg1 =finalizeMsg1;
        String finalizeMsg2 = sg.getFinalizeMsg2();
        
        String requestForNextMessage ="<Alert>"+"\n"+
                   "<CmdID>2</CmdID>"+"\n"+
                   "<Data>222</Data>"+"\n"+
                   "</Alert>";
        
        String serverResponse = null;
        boolean done                       = false;
        
      
       finalizeMsg1 = replaceTag(finalizeMsg1,"{6}","");  
       finalizeMsg1 = replaceTag(finalizeMsg1,"{7}",requestForNextMessage);  
       finalizeMsg1 = replaceTag(finalizeMsg1,"{8}",msgCount+"");   
       finalizeMsg1 = replaceTag(finalizeMsg1,"{9}",msgRefCount+"");
            
       serverResponse = sendSyncMessage("finalizeMsg1",finalizeMsg1);
      
        done = 
                ((serverResponse.indexOf("<Final/>") >= 0) 
                || (serverResponse.indexOf("</Final>") >= 0));
        
        stringStatusBuffer = new StringBuffer();           
        processNode(serverResponse);  
        msgCount++;
        msgRefCount++;
        
       if(done)
       {
         requestForNextMessage ="";//no next messages pending  
       }   
       
       while(!done)
       {  finalizeMsg1 =backupFinalizeMsg1;
          finalizeMsg1 = replaceTag(finalizeMsg1,"{6}",stringStatusBuffer.toString());  
          finalizeMsg1 = replaceTag(finalizeMsg1,"{7}",requestForNextMessage);  
          finalizeMsg1 = replaceTag(finalizeMsg1,"{8}",msgCount+"");   
          finalizeMsg1 = replaceTag(finalizeMsg1,"{9}",msgRefCount+"");
       
          StaticDataHelper.log("finalizing---1"); 
          StaticDataHelper.log(finalizeMsg1);
        
          serverResponse= sendSyncMessage("finalizeMsg1",finalizeMsg1);
           
          stringStatusBuffer = new StringBuffer();
          processNode(serverResponse);
                
        
       done = 
                ((serverResponse.indexOf("<Final/>") >= 0) 
                || (serverResponse.indexOf("</Final>") >= 0));
        
        
        msgCount++; 
        msgRefCount++;
       }
       
            
           
            finalizeMsg2 = replaceTag(finalizeMsg2,"{8}",msgCount+"");
            finalizeMsg2 = replaceTag(finalizeMsg2,"{9}",msgRefCount+"");
            finalizeMsg2 = replaceTag(finalizeMsg2,"{6}",stringStatusBuffer.toString());  
            
            serverResponse = sendSyncMessage("finalizeMsg2",finalizeMsg2);
         
            //finally store the time stamp     
           storeTimestamp();    
                  
    }
    
    
    /**
    * generates syncml for slowSync 
    * @param   String  :serverResponse creates status for all server response 
    */    
    private void processNode(String serverResponse){
            
            String processNode = serverResponse;
            
            int commandID = 3; 
            
            while(processNode.indexOf("</Add>") > 0){
                    
                    String mail = getXMLTagValue(processNode,"Add");
                    String folderName = getXMLTagValue(mail,"LocURI");
                    String commandId = getXMLTagValue(mail,"CmdID");
                    if(parseMailMessage(mail,folderName,"Add") == 0){
            
                        buildStatusMsg(commandId,"Add",folderName,"201",commandID);
            
                    }
                    
                    else{
                    
                        buildStatusMsg(commandId,"Add",folderName,"500",commandID);
                    
                    }
                    
                    commandID++;
                    
                    processNode = processNode.substring(processNode.indexOf("</Add>") + 6,processNode.length());
            }
            
            processNode = serverResponse;
            
            while(processNode.indexOf("</Delete>") > 0){
                
                String mail = getXMLTagValue(processNode,"Delete");
                String folderName = getXMLTagValue(mail,"LocURI");
                String commandId = getXMLTagValue(mail,"CmdID");
                
                if(parseMailMessage(mail,folderName,"Delete") == 0){
                
                    buildStatusMsg(commandId,"Delete",folderName,"200",commandID);
                
                }
                else{
                
                    buildStatusMsg(commandId,"Delete",folderName,"500",commandID);
                
                }
                
                commandID++;
                
                processNode = processNode.substring(processNode.indexOf("</Delete>") + 9,processNode.length());
            }
            
        }
         
    /**
    * stores the current timestamp if sync is successful
    */          
        public void storeTimestamp(){
                
            store = PersistentStore.getPersistentObject(CONFIG_KEY);    
            
            synchronized(store){
                
                String [] timeStamparr = {new String(new Long(timeStamp).toString())};
                store.setContents(timeStamparr);
                store.commit();
            }
               
        }
        
    /**
    * replaces a  variable data in xml
    * @param   String  :msgXML passed syncml
    * @param   String  :variable  to be searched
    * @param   String  :variableValue value of variable to be replaced
    * @return  String  :same xml with replaced variable value
    */         
        public String replaceTag(String msgXML, String variable, String variableValue) {

            String msgXMLBefore = null;
            
            String msgXMLAfter  = null;
            
            while(msgXML.indexOf(variable) >= 0){
                
                msgXMLBefore = msgXML.substring(0, msgXML.indexOf(variable));
                msgXMLAfter  = msgXML.substring(msgXML.indexOf(variable) + variable.length());
                msgXML = msgXMLBefore + variableValue + msgXMLAfter;
            
            }
            return msgXML;
        
        }
        
        
    /**
    * actually stores or deletes messages got from server
    * @param   String  :mail passed syncml
    * @param   String  :locURI  the type of folder
    * @param   String  :function type of command
    * @return  int     :0 on success or -1 on failure
    */   
        private int parseMailMessage(String mail, String locURI,String function){
        
            Folder folder = null;
            String flag = " ";
            
            try{
                if(locURI.startsWith("I")){
            
                    folder = Session.waitForDefaultSession().getStore().getFolder("Inbox");
            
                }
            
                if(locURI.startsWith("O")){
            
                    folder = Session.waitForDefaultSession().getStore().getFolder("Outbox");
            
                }
                if(locURI.startsWith("D")){
            
                    folder = Session.waitForDefaultSession().getStore().getFolder("Inbox");
                    flag= "save";
            
                }
                
                if(locURI.startsWith("T")){
                
                        folder = Session.waitForDefaultSession().getStore().getFolder("Inbox");
                        flag = "trash";
                
                }else if(locURI.startsWith("S")){
                
                    Folder[] folders = Session.waitForDefaultSession().getStore().list(Folder.SENT);
                    folder = folders[0];
                    flag = "sent";
                
                }
              
                if(folder != null){
                
                    if(function.equals("Add")){
                    
                        mail = replaceTag(mail,"&lt;","<"); // to be discussed with funambol
                    
                        String[] messageID = new StringUtil().split(locURI,"/");
                        MessageIDStore messageIDStore = new MessageIDStore();
                        if(messageIDStore.checkIsDataThere(locURI) || checkIsDataThere(messageID[1],Constants.ITEMSKEY)){
                        
                            return 0;
                        
                        }else{
                            
                            Message msg = mailParser.parseMessage(mail,locURI);
                            folder.appendMessage(msg);
                        
                            if(flag.equals("save")){
                        
                                msg.setFlag(Message.Flag.SAVED, true);
                        
                            }
                            
                            messageIDStore.add(locURI,new Integer(msg.getMessageId()).toString());
                                              
                            appendData(messageID[0]+":"+new Integer(msg.getMessageId()).toString(),Constants.ITEMSKEY);
                    
                            folder = null;
                    }
                
                    return 0;
                
                }else if(function.equals("Delete")){
                    
                    String[] serverMessageID = new StringUtil().split(locURI,"/");
                    
                    MessageIDStore messageIDStore = new MessageIDStore();
                
                    int messageID;
                
                    String messageIDfromRS = messageIDStore.getClientId(locURI);
                
                    if(messageIDfromRS == null){
                    
                        messageID = Integer.parseInt(serverMessageID[1]);
                    
                     }else{
                    
                        messageID = Integer.parseInt(messageIDfromRS);
                        messageIDStore.delete(locURI);
                    }
                    
                    Message todelete = Store.getMessage(messageID);
                    
                    if(todelete != null){
                    
                        folder.deleteMessage(todelete);
                        deleteData(new Integer(todelete.getMessageId()).toString(),Constants.ITEMSKEY);
                    
                    }
                    else{
                    
                       return -1;
                    
                    }
                   
                   folder = null;
                   return 0;
                 
                 
                 }
               
               }
            
            }catch(Exception e){
                StaticDataHelper.log(e.getMessage());
            }
        return -1;
    }
        
    /**
    * checks for server alerts and parses corrsponding codes 
    * @param   int  :alertCode passed syncml
    * @return  int  :code for corresponding alertCode or 0
    */       
    public int checkforAlertCodes(int alertCode ){
              
        if (alertCode == ALERT_CODE_SLOW) {
    
            return ALERT_CODE_SLOW;
    
        } else if (alertCode == ALERT_CODE_REFRESH_FROM_SERVER) {
    
            return ALERT_CODE_REFRESH_FROM_SERVER;
    
        } else if (alertCode == ALERT_CODE_ONE_WAY_FROM_SERVER) {
            //send empty sync
            return ALERT_CODE_ONE_WAY_FROM_SERVER;
        
        } else if (alertCode == ALERT_CODE_FAST){
            //fast sync        
            return ALERT_CODE_FAST;
        
        } else {
        
            return 0;
        
        }
    }
          
    /**
    * entry function for all processing
    * throws Exception
    */  
    public void processEmail()throws Exception{
        
        int alertCode = 0;
        
        try{
        
            Configuration.updateConfigdetails();
            int alert= sendAuthenticationMessage();
        
            if(alert != -1){
        
                alertCode=checkforAlertCodes(alert);  
        
             }else{
        
                throw new SyncException( "The server did not sent an Alert for "   +
                Configuration.PROTOCOL                                +
                ". The source will not be synchronized."
                );
            }
            
            sendClientMailSyncMessage(alertCode);
            StaticDataHelper.log("**********Finalizing client to server  sync***********");
            finalizeSync();
            
            }catch(AuthenticationException ae) {
            
                 StaticDataHelper.log(getSynchError(ae.getMessage()));
                 throw ae;
            
            }catch(SyncException se){
            
                StaticDataHelper.log(getSynchError(se.getMessage()));
                throw se;
            
            }catch(IOException ioe) { 
            
                StaticDataHelper.log(ioe.getMessage());
                throw ioe;
            
            }catch(Exception e){
            
                StaticDataHelper.log(getSynchError(e.getMessage()));
                throw e;
            
            }
        }
        
    /**
    * stores server sent alerts to hashtable 
    * @param   String  :msg type i.e add , delete etc. 
    * @throws SyncException
    * @throws AuthenticationException
    */     
        private void checkServerAlerts(String msg)
             throws SyncException, AuthenticationException {

            Vector xmlMsg    = null;
            Vector itemMsg   = null;
            Vector targetTag = null;
    
            String alert     = null;
            String item      = null;
            String dataStore = null;
    
            xmlMsg = new Vector();
            xmlMsg.addElement(msg);
    
            serverAlerts = new Hashtable();
    
            Vector alertTag = getXMLTag(getXMLTag(getXMLTag(xmlMsg, TAG_SYNCML),TAG_SYNCBODY), TAG_ALERT);
    
            int l = alertTag.size();

            for (int i=0; i < l; i++) {
                itemMsg = new Vector();
    
                alert = getXMLTagValue((String) alertTag.elementAt(i), TAG_DATA);
    
                item = getXMLTagValue((String) alertTag.elementAt(i), TAG_ITEM);
    
                itemMsg.addElement(item);
    
                targetTag = getXMLTag(itemMsg, TAG_TARGET);
    
                int m = targetTag.size();
    
                for (int j=0; j < m; j++) {
                    
                    dataStore = getXMLTagValue((String) targetTag.elementAt(j), TAG_LOC_URI);
                
                }

                StaticDataHelper.log("The server alert code for " + dataStore + " is " + alert);

                serverAlerts.put(dataStore, alert);
        }
    
    }
         
    /**
    * returns a vector of particular tag inside another vector
    * @param   Vector  :xmlInput passes xml
    * @param   String  :tag  the tag the syncml for which is to be returned from xmlInput
    * @return  Vector  :the vector of tags 
    * @throws SyncException
    */        
    private Vector getXMLTag(Vector xmlInput, String tag)
                throws SyncException {

        Vector xmlReturn    = null ;
        String xmlInputTag  = null ;
        String endTag       = null ;

        endTag   = "</" + tag + ">" ;

        xmlReturn = new Vector();

        try {

            for (int j=0, l = xmlInput.size(); j < l; j++) {

                xmlInputTag = (String) xmlInput.elementAt(j);

                //
                // tag without namespace
                // or tag with namespace
                //
                while (xmlInputTag.indexOf("<" + tag + ">") != -1 ||
                       xmlInputTag.indexOf("<" + tag + " ") != -1) {

                    xmlReturn.addElement(getXMLTagValue(xmlInputTag, tag));

                    xmlInputTag =
                        xmlInputTag.substring(
                                        xmlInputTag.
                                            indexOf(endTag) + endTag.length());

                }

            }

        } catch (Exception e) {
             StaticDataHelper.log("Parsing XML error, TAG " +
                                    tag                       +
                                    " - : "                   +
                                    e.getMessage()          );
            throw new SyncException("Parsing XML error, TAG " +
                                    tag                       +
                                    " - : "                   +
                                    e.getMessage()            );
                                    
                                   
        }

        return xmlReturn;

    }      
       
         
    /**
    * returns the alert code for source database
    * @param   String  :sourceURI database name
    * @return  int  :code or error code
    */    
    private int getSourceAlertCode(String sourceURI) {
            
        try {
            
            return Integer.parseInt((String)serverAlerts.get(sourceURI));
        
        } catch (Throwable t) {
    
                StaticDataHelper.log  ( "ERROR: unrecognized server alert code ("
                                + serverAlerts.get(sourceURI)
                                        + ") for "
                                    + sourceURI
                                );
        }
    
        return -1;
    }

    
     /**
    * checks the code for status 
    * @param   String  :msg status syncml
    * @return  String  :statusOf command
    * @throws SyncException
    * @throws AuthenticationException
    * @throws UpdateException    
    */ 
    private void checkStatus(String msg, String statusOf)
        throws SyncException, AuthenticationException, UpdateException {

        Vector xmlMsg = new Vector();
        xmlMsg.addElement(msg);
        Vector statusTag = getXMLTag(getXMLTag(getXMLTag(xmlMsg, TAG_SYNCML),TAG_SYNCBODY), TAG_STATUS);
        int l = statusTag.size();
        
        for (int i=0; i < l; i++) {
        
                if (getXMLTagValue((String) statusTag.elementAt(i), TAG_CMD).equals(statusOf)) {
                        String statusCode = getXMLTagValue((String) statusTag.elementAt(i), TAG_DATA);

                if (String.valueOf(STATUS_CODE_OK).equals(statusCode)) {
                        //
                        // 200
                        //
                        return;
                }
                else if (String.valueOf(STATUS_CODE_FINE).equals(statusCode)){
                        //fine
                        return;   
                }
                else if (String.valueOf(STATUS_REFRESH_REQUIRED).equals(statusCode)) {
                        //
                        // 508
                        //
                    return;
                } else if (String.valueOf(STATUS_CODE_AUTHENTICATION_ACCEPTED).equals(statusCode)) {
                        //
                        // 212
                        //
                    return;
                } else if (String.valueOf("500").equals(statusCode)) {
                        //
                        // 500
                        //
                        return;
                } else if (String.valueOf(STATUS_CODE_FORBIDDEN).equals(statusCode)) {
                        //
                        // 401
                        //
                    StaticDataHelper.log("Sorry, you are not authorized to synchronize "
                            + Configuration.PROTOCOL
                            + ". Go to the preferences panel and make sure your account settings are correct."
                    );
                    throw new AuthenticationException(
                            "Sorry, you are not authorized to synchronize "
                            + Configuration.PROTOCOL
                            + ". Go to the preferences panel and make sure your account settings are correct."
                        );
                } else {
                        //
                        // Unhandled status code
                        //

                    throw new UpdateException(statusCode);
                }
            }

        }  // next i

        String msgE = msg;
        throw new SyncException(msgE);
    }    
    
} 

