/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package funambol.syncclient.blackberry.email.impl;

import funambol.syncclient.util.StaticDataHelper;
import funambol.syncclient.blackberry.Configuration;
import java.io.IOException;
import funambol.syncclient.common.Base64;
import funambol.syncclient.spds.*;
import java.util.Hashtable;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import net.rim.device.api.system.DeviceInfo;

/**
 * <code>SyncGen</code> helper class for generating syncml
 *
 * $Id: SyncGen.java,v 1.0
 */
 
 public class SyncGen {
   
    private StringBuffer sb;
    private static PersistentObject store;
    public static long   CONFIG_KEY     = 0xb5651L;

    private final String XML_INITIALIZATION   = "/xml/mailinit.xml" ;
    private final String XML_SYNC             = "/xml/mailsync.xml"  ;
    private final String XML_FINALIZE1        = "/xml/finalizesync1.xml"  ;
    private final String XML_FINALIZE2        = "/xml/finalizesync2.xml"  ;
    
    private long lastTimestamp;
    private static final String SESSION_ID           = "12345678";
    
    private static final String DEVICE_ID_PREFIX     = "fbb-" ;

    private String      serverUrlStatus       = null ;
    private String      gatewayApn            = null ;
    private String      gatewayIp             = null ;
    private String      gatewayPort           = null ;
    private String      gatewayUrl            = null ;

    private String      login                 = null ;

    private Hashtable   mappings              = null ;


    private String      clientInitXML         = null ;
    private String      modificationsXML      = null ;
    private String      mappingXML            = null ;
    private String      mailXML               = null ;
    private String      finalizeXML1          = null ;
    private String      finalizeXML2          = null ;
    
    private String      deviceId              = null ;
    
    private static final String TAG_ALERT            = "Alert"         ;
    private static final String TAG_ADD              = "Add"           ;
    private static final String TAG_CMD              = "Cmd"           ;
    private static final String TAG_DATA             = "Data"          ;
    private static final String TAG_DELETE           = "Delete"        ;
    private static final String TAG_ITEM             = "Item"          ;
    private static final String TAG_LOC_URI          = "LocURI"        ;
    private static final String TAG_REPLACE          = "Replace"       ;
    private static final String TAG_STATUS           = "Status"        ;
    private static final String TAG_SYNC             = "Sync"          ;
    private static final String TAG_SYNCBODY         = "SyncBody"      ;
    private static final String TAG_SYNCHDR          = "SyncHdr"       ;
    private static final String TAG_SYNCML           = "SyncML"        ;
    private static final String TAG_TARGET           = "Target"        ;
    public static long nextTimeStamp = 0;
    public void gen(){
    }
    
    public  SyncGen() {
        deviceId = DEVICE_ID_PREFIX + String.valueOf(DeviceInfo.getDeviceId());  
    }

    
    /**
    * loads the various xml 
    * @throws IOException
    */ 
    public void loadResources() throws IOException {
        
        Class c = this.getClass();

        clientInitXML = StaticDataHelper.read(
            c.getResourceAsStream(XML_INITIALIZATION)
        );

        mailXML =   StaticDataHelper.read(
            c.getResourceAsStream(XML_SYNC)
        );
       
        finalizeXML1 = StaticDataHelper.read(
            c.getResourceAsStream(XML_FINALIZE1)
        );

        finalizeXML2 = StaticDataHelper.read(
            c.getResourceAsStream(XML_FINALIZE2)
        );

        prepareFinalizeMessages();
        prepareInizializationMessage();

    }

    /**
    * returns  xml for intial messages
    */ 
    
    
    public String getInit(){
        return clientInitXML;
    }


    
    /**
    * returns  xml for modified messages
    */ 
    
    
    public String getMod(){
        return modificationsXML;
    }
    
    
    /**
    * returns  xml for mapped messages
    */ 
    
     
    public String getMap(){
        return mappingXML; 
    }

    
    /**
    * returns  xml for mail messages
    */ 
    public String getMail(){
     return mailXML; 
    }
    
    
    /**
    * returns  xml for finalize1 messages
    */ 
     
    public String getFinalizeMsg1(){
        return finalizeXML1;
    }
    
    
   
    /**
    * returns  xml for finalize2 messages
    */ 
    
    public String getFinalizeMsg2(){
        return finalizeXML2;
    }
    
    
    /**
    * prepares the initial syncml
    */
     
    private void prepareInizializationMessage(){

        String dbAlertsXML = null ;
        dbAlertsXML    = createAlerts(Configuration.PROTOCOL)   ;

        //
        // authentication basic
        //
        login = Configuration.USERNAME+":"+Configuration.PASSWORD;
        login = new String(Base64.encode(login.getBytes()));

        clientInitXML  = messageFormat(clientInitXML, "{0}", SESSION_ID  );
        clientInitXML  = messageFormat(clientInitXML, "{1}", Configuration.URL   );
        clientInitXML  = messageFormat(clientInitXML, "{2}", deviceId);
        clientInitXML  = messageFormat(clientInitXML, "{3}", login       );
        clientInitXML  = messageFormat(clientInitXML, "{4}", dbAlertsXML );
        clientInitXML  = messageFormat(clientInitXML, "{5}", Configuration.MAXMSGSIZE );

    }
    
    
   /**
    * returns  the lasttimestamp
    * @return String :lasttimestamp
    */
    
     
    public String getLasttime(){

        store = PersistentStore.getPersistentObject(CONFIG_KEY);
        
        String [] lastTimeStamp=(String []) store.getContents();
        
        if(lastTimeStamp!=null){
            return lastTimeStamp[0];
        }
        
        return null;

    }
    
    
    /**
    * returns client mail syncml
    * @throws Exception
    */
    
     
    public String createMailMessage()
            throws SyncException{

       // String timestamp ="<Next>" + nextTimeStamp + "</Next>\n";
        
       
        mailXML=messageFormat(mailXML, "{0}", SESSION_ID                 );
        mailXML=messageFormat(mailXML, "{1}", Configuration.URL          );
        mailXML=messageFormat(mailXML, "{2}", Configuration.URL          );
        mailXML=messageFormat(mailXML, "{3}", deviceId                   );
        mailXML=messageFormat(mailXML, "{4}", deviceId                   );
        mailXML=messageFormat(mailXML, "{6}", Configuration.PROTOCOL     );
        mailXML=messageFormat(mailXML, "{7}", Configuration.PROTOCOL     );
        mailXML=messageFormat(mailXML, "{9}", Configuration.MAXMSGSIZE   );

        return mailXML;
    }
    
    
    /**
    * prepares the syncml for finalize messages 
    */
    
     
    private void prepareFinalizeMessages(){
       
        finalizeXML1 = messageFormat(finalizeXML1, "{0}", SESSION_ID  );
        finalizeXML1 = messageFormat(finalizeXML1, "{1}", Configuration.URL  );
        finalizeXML1 = messageFormat(finalizeXML1, "{2}", Configuration.MAXMSGSIZE  );
        finalizeXML1 = messageFormat(finalizeXML1, "{3}", Configuration.URL );
        finalizeXML1 = messageFormat(finalizeXML1, "{4}", deviceId          );
        finalizeXML1 = messageFormat(finalizeXML1, "{5}", deviceId );
        finalizeXML2 = messageFormat(finalizeXML2, "{0}", SESSION_ID  );
        finalizeXML2 = messageFormat(finalizeXML2, "{1}", Configuration.URL  );
        finalizeXML2 = messageFormat(finalizeXML2, "{2}", Configuration.MAXMSGSIZE  );
        finalizeXML2 = messageFormat(finalizeXML2, "{3}",  Configuration.URL );
        finalizeXML2 = messageFormat(finalizeXML2, "{4}",  Configuration.PROTOCOL );
        finalizeXML2 = messageFormat(finalizeXML2, "{5}",  Configuration.PROTOCOL );
        finalizeXML2 = messageFormat(finalizeXML2, "{10}",  deviceId );
        finalizeXML2 = messageFormat(finalizeXML2, "{11}",  deviceId );
    }
    
    
    /** search and replace a variable with passed value
    * @param String  :msgXML this xml passed
    * @param String  :variable to be searched and replaced 
    * @param String  :variableValue replace value
    * @return String :replaced syncml
    */
    
     
    public String messageFormat(String msgXML, String variable, String variableValue) {

        String msgXMLBefore = null;
        String msgXMLAfter  = null;

        msgXMLBefore = msgXML.substring(0, msgXML.indexOf(variable));
        msgXMLAfter  = msgXML.substring(msgXML.indexOf(variable) + variable.length());

        return (msgXMLBefore + variableValue + msgXMLAfter);

    }
    
    /**
    * creates alerts for passed source url  
    * @param String  :sourceUri this the database 
    * @return String :alert syncml
    */
    
     
    public String createAlerts(String sourceUri) {

        StringBuffer sb = new StringBuffer();

        //change this line later

        String timestamp ="<Next>" + nextTimeStamp + "</Next>\n";
        
        if(getLasttime()!=null){
            lastTimestamp =Long.parseLong(getLasttime());
            timestamp="<Last>" + lastTimestamp + "</Last>\n" + timestamp;
        }
        
        sb.append("<Alert>\n");
        sb.append("<CmdID>1</CmdID>\n");
        sb.append("<Data>");
        if(getLasttime()!=null){
            sb.append("200");
        }else{
            sb.append("201");
        }
        sb.append("</Data>\n");
        sb.append("<Item>\n");
        sb.append("<Target><LocURI>");
        sb.append(sourceUri);
        sb.append("</LocURI></Target>\n");
        sb.append("<Source><LocURI>");
        sb.append(sourceUri);
        sb.append("</LocURI></Source>\n");
        sb.append("<Meta>\n");
        sb.append("<Anchor xmlns=\"syncml:metinf\">\n");
        sb.append(timestamp);
        sb.append("</Anchor>\n");
        sb.append("</Meta>\n");
        sb.append("</Item>\n");
        sb.append("</Alert>");
        sb.append("\n");
        return sb.toString();
    }
}


