/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package funambol.syncclient.blackberry.email.impl;

import net.rim.blackberry.api.mail.Folder;
import net.rim.blackberry.api.mail.Message;
import net.rim.blackberry.api.mail.Address;
import net.rim.blackberry.api.mail.TextBodyPart;
import java.util.*;
import funambol.syncclient.common.StringTools;
import funambol.syncclient.blackberry.Configuration;
import funambol.syncclient.util.StaticDataHelper;

/**
 * <code>XMLMailParser</code> Implementation of Mail Parsing.
 *
 * 
 * $Id: XMLMailParser.java,v 1.2 2006/02/03 08:27:39 fabius Exp $
 */
public class XMLMailParser implements MailParser {
    
    private String tagArray[]={"read","replied","modified","received","created","flagged","deleted","emailitem","forwarded"};
    private static final String START_MARKER ="<Email>";
    private static final String END_MARKER = "</Email>";
    private static final String UTC_TIMEZONE = "UTC";
    private static Hashtable h;
    private Enumeration em;
    private MIMEParser mp;
    private static final String READ ="read";
    private static final String REPLIED="replied";
    private static final String MODIFIED ="modified";
    private static final String RECIEVED="recieved";
    private static final String CREATED="created";
    private static final String FLAGGED="flagged";
    private static final String DELETED="deleted";
    private static final String FORWARDED="forwarded";
   
    private Message mailmessage;
    private String foldername;
    private int commandId = 4;
    
    
    /**
    *
    *Constructor
    *
    **/
    
    public XMLMailParser()
    {    
        mp = new MIMEParser();
    }
    
    /**
    *
    *Returns the commandID
    *
    **/
    
    public int getCommandId(){
        
        return commandId++;
    
    }
    
    public void setCommandId(int commandId ){
    
        this.commandId = commandId;    
    }
    
    /**
    *
    *Sets the parse params
    *
    **/
    public void setParseParams(String fname, Message message){
    
        this.mailmessage = message;
        this.foldername = fname;
    
    }
    
    
    /**
    *
    *Method to parse the message String & build the message object
    *
    **/
    
    public Message parseMessage(String messageString,String locURI){
        //here we get a syncml string for email and need to create the Message 
        Message msg = null;
        String toParse =messageString;
        h=buildMapFromXML(toParse);
        
        if(h!=null){
            //you have got a mail!    
            String emailitem= mimedata(h);
            msg=mp.parseMessage(emailitem,locURI);   
            
            String field =getValue(h,READ);
            addBooleanField(msg,field,READ);
            field =getValue(h,FORWARDED);
            addBooleanField(msg,field,FORWARDED);
            field =getValue(h,REPLIED);
            addBooleanField(msg,field,REPLIED);
            field = getValue(h,RECIEVED);
            addDateField(msg,field,RECIEVED);
            field = getValue(h,MODIFIED);
            addDateField(msg,field,MODIFIED);
            field = getValue(h,CREATED);
            addDateField(msg,field,CREATED);
            field = getValue(h,DELETED);
            addBooleanField(msg,field,DELETED);
            field = getValue(h,FLAGGED);
            addBooleanField(msg,field,FLAGGED);
        
        }
    
        return msg;
    }
    
    /**
    *
    *Method to parse the Message object & builds the Message String
    *
    **/
    
    private String parseMessageObject() throws Exception{
       
        StringBuffer messageBuffer = new StringBuffer();
               
        messageBuffer.append("<emailitem>" + "\n" + "<![CDATA[");
        ///
        MessageIDStore msgIdStore = new MessageIDStore();
        String messageIDfromStore = msgIdStore.getServerId(new 
                    Integer(mailmessage.getMessageId()).toString());
                
        
        if(messageIDfromStore != null){
            
            msgIdStore.setMessageMappingKey();
            
            if(msgIdStore.getClientId(messageIDfromStore) != null){
                
                messageBuffer.append("Message-ID: "+msgIdStore.getClientId(messageIDfromStore)+"\r\n"); 
            
            }else{
            
               messageBuffer.append("Message-ID: "+mailmessage.getMessageId()+"\r\n"); 
            
            }
            
        }else{
        
            messageBuffer.append("Message-ID: "+mailmessage.getMessageId()+"\r\n");

        }
        
         ///   
       
        
        if(mailmessage.getFrom()  != null){

            messageBuffer.append("From: "+mailmessage.getFrom().getAddr()+"\r\n");

        }else{
            
            messageBuffer.append("From: "+Configuration.FROM+"\r\n");
        
        }
        messageBuffer.append("To: ");
    
        Address[] addressList=mailmessage.getRecipients(Message.RecipientType.TO);
        
        for(int i=0;i<addressList.length;i++){
        
            if(i > 0){
                messageBuffer.append(";");
            }
            
            messageBuffer.append(addressList[i].getAddr());
        }
    
        messageBuffer.append("\r\n");
        
        messageBuffer.append("Subject: "+mailmessage.getSubject()+"\r\n");
        
        messageBuffer.append("Content-Type: "+mailmessage.getContentType()+"\r\n");
        
        messageBuffer.append("Date: "+mailmessage.getSentDate().toString()+"\r\n");
        
        messageBuffer.append("\r\n");//end of headers
        
        if((TextBodyPart)mailmessage.getContent() != null)
                messageBuffer.append(((TextBodyPart)mailmessage.getContent()).getContent());
        
        messageBuffer.append("\r\n\r\n"+"]]&gt;");
        messageBuffer.append("\n"+"</emailitem>");
    
        return messageBuffer.toString();
     }
    
    /**
    *
    *Converts the date to string
    *
    **/
    
    private String convertFromDatetoString()
    {
        StringBuffer date = new StringBuffer();
        Calendar cal = Calendar.getInstance();
        cal.setTime(mailmessage.getSentDate());
        
        date.append(cal.get(Calendar.YEAR));
        
        if((cal.get(Calendar.MONTH) + 1) > 9){
            date.append(cal.get(Calendar.MONTH) + 1);
        }else{
            date.append("0"+cal.get(Calendar.MONTH));
        }
        if(cal.get(Calendar.DATE)  > 9)  {
            date.append(cal.get(Calendar.DATE));
        }else{
            date.append("0"+cal.get(Calendar.DATE));
        } 
        date.append("T");
        
        if(cal.get(Calendar.HOUR) > 9){
            date.append(cal.get(Calendar.HOUR));
        }else{ 
            date.append("0"+cal.get(Calendar.HOUR));
        }
        if(cal.get(Calendar.MINUTE) > 9){
            date.append(cal.get(Calendar.MINUTE));
        }else{
            date.append("0"+cal.get(Calendar.MINUTE));  
        }
        
        if(cal.get(Calendar.SECOND) > 9){
            
            date.append(cal.get(Calendar.SECOND));
        }else{ 
            date.append("0"+cal.get(Calendar.SECOND));  
        }
        date.append("Z");
        return date.toString();
    }
    
    /**
    *
    *Method to convert entire email data to string
    *
    **/
    
    public String toString()
    {
        StringBuffer messageBuffer = new StringBuffer();
        
       
        messageBuffer.append("<Add>"+"\n");
        messageBuffer.append("<CmdID>"+commandId+"</CmdID>"+"\n");
        messageBuffer.append("<Meta>"+"\n");
        messageBuffer.append("<Type xmlns='syncml:metinf'>application/vnd.omads-email+xml</Type>"+"\n");
        messageBuffer.append("</Meta>"+"\n");
        messageBuffer.append("<Item>"+"\n");
        messageBuffer.append("<Source>"+"\n");
        messageBuffer.append("<LocURI>");   
        
        String messageIDfromStore = new MessageIDStore().getServerId(new Integer(mailmessage.getMessageId()).toString());
                
        if(messageIDfromStore != null){
            
            messageBuffer.append(messageIDfromStore);
        
        }else{
            
            messageBuffer.append(foldername+"/"+ mailmessage.getMessageId());
        
        }
        
        
        messageBuffer.append("</LocURI>"+"\n");        
        messageBuffer.append("</Source>"+"\n");  
        messageBuffer.append("<Data>"+"\n");  
        messageBuffer.append("<![CDATA[<Email>");                    
        messageBuffer.append("\n");                          
        messageBuffer.append("<read>true</read>"+"\n");
        messageBuffer.append("<received>"+convertFromDatetoString()+"</received>"+"\n");
        messageBuffer.append("<created>"+convertFromDatetoString()+"</created> "+"\n");                              
        
        try{ 
            messageBuffer.append(parseMessageObject());
        }catch(Exception e){}
        
        messageBuffer.append("\n");
        messageBuffer.append("</Email>]]>");
        messageBuffer.append("\n"); 
        messageBuffer.append("</Data>"+"\n");
        messageBuffer.append("</Item>"+"\n");
        messageBuffer.append("</Add>");
        
 
        commandId++;
        return messageBuffer.toString();
    }
    
    /**
    *
    *Returns the value for a key from the hashtable
    *
    **/
     
    private String getValue(Hashtable eventMap, String key) {
        Object val = eventMap.get(key);
        String value = val == null ? "" : val.toString();
        return value.trim();
    }
    
    /**
    *
    * This method creates a hashtable with keys and values.
    *
    **/
    
    private Hashtable buildMapFromXML(String messageString){
        int startMarkerIndex = messageString.indexOf(START_MARKER);
        int endMarkerIndex = messageString.indexOf(END_MARKER);
        
        if(startMarkerIndex == -1 || endMarkerIndex == -1){
        
            StaticDataHelper.log("Improper data from the server.");
            return null;
        
        }
        
        Hashtable eventMap = new Hashtable();
        String startTag = "";
        String endTag = "";
        String degeneratedTag = "";
        String data = "";
        for(int i = 0; i < tagArray.length; i++){
            
            String tag = tagArray[i];
            startTag = "<" + tag + ">";
            int startTagIndex = messageString.indexOf(startTag);
            
            if(startTagIndex == -1){
                continue;
            }
            
            degeneratedTag = "<" + tag + "/>";
            int degeneratedTagIndex = messageString.indexOf(degeneratedTag);
            
            if(degeneratedTagIndex != -1){
                continue;
            }
            
            endTag = "</" + tag + ">";
            
            int endTagIndex = messageString.indexOf(endTag);
            
            if(endTagIndex != -1){
                data = messageString.substring(startTagIndex + startTag.length(), endTagIndex);
            }else{
                data = "";
            }
            if(data.length() > 0){
                data = StringTools.unescapeXml(data);
            }
            eventMap.put(tag, data);
        }

        return eventMap;
    }
    
    /**
    *
    * Check the flag for the message
    *
    **/
    
    private void addBooleanField(Message msg,String field ,String name){
 
        if(!field.equals("")){
            
            boolean b=field.equals("true")?true:false;
            
            if(name.indexOf("read")>=0){   
                
                msg.setFlag(Message.Flag.OPENED,b);
            
            }else if(name.indexOf("deleted")>=0){
            
                msg.setFlag(Message.Flag.DELETED,b);    
            
            } 
         
        }  
    }
 
    /**
    *
    * Adds the date field to the message header
    *
    **/
    
    public void addDateField(Message msg ,String field , String name){
        
        if(!field.equals("")){      
            msg.addHeader(name+"date",field);
        } 
        
    }
    
    /**
    *
    * Returns the value of emailitem
    *
    **/
    private String mimedata(Hashtable h){
        String valuemime="";
        Enumeration emime = h.keys();
        while(emime.hasMoreElements()){
        
            Object keyobj=emime.nextElement();   
            String keymime=keyobj.toString();
            if(keymime.indexOf("emailitem")>=0){
                valuemime= (String)h.get(keyobj);
            }     
        }
        
        return valuemime;
    }
} 
